program Show_BMP_File;
{****************************************************************************
** Demonstration of loading a BMP file and displaying it on screen         **
**  by Steven H Don                                                        **
**                                                                         **
** This support Windows' 256 colour BMP files with a maximum resolution    **
** of 320x200.                                                             **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************}

uses Dos, Crt;

type BMPHeader = record
  bfType           : Word;
  bfSize,
  bfReserved,
  bfOffBits,
  biSize,
  biWidth,
  biHeight         : LongInt;
  biPlanes,
  biBitCount       : Word;
  biCompression,
  biSizeImage,
  biXPelsPerMeter,
  biYPelsPerMeter,
  biClrUsed,
  biClrImportant   : LongInt;
end;

type BufferType = Array [0..63999] of Byte;

var
  {An offscreen buffer}
  Buffer        : ^BufferType;
  FileName      : String;
  Palette       : Array [0..255, 0..2] of Byte;
  x, y          : Integer;

{This sets the display to VGA 320x200 in 256 colours}
procedure VGAScreen; assembler;
asm
  mov ax, 13h
  int 10h
end;

{This resets the display to text mode}
procedure TextScreen; assembler;
asm
  mov ax, 3h
  int 10h
end;

{This sets a DAC register to a specific Red Green Blue-value}
procedure SetDAC(DAC, R, G, B : Byte);
begin
  Port[$3C8] := DAC;
  Port[$3C9] := R;
  Port[$3C9] := G;
  Port[$3C9] := B;
end;

{Assigns memory for a buffer}
function MakeBuffer : Boolean;
begin
  {Check if enough memory is available}
  if MaxAvail < 64000 then begin
    {If not, return false}
    MakeBuffer := false;
    exit;
  end;
  New (Buffer);
end;

{Clears the buffer}
procedure ClearBuffer;
begin
  fillchar (Buffer^, 64000, 0);
end;

{Displays the buffer}
procedure DisplayBuffer;
begin
  move (Buffer^,          {from buffer}
        Mem [$A000:0000], {to screen}
        64000);           {64000 pixels}
end;

{This loads in BMP File and stores it in the buffer.}
procedure LoadBMP(f : String);
var
  OffSet, Lines, Counter         : Word;
  Header                         : BMPHeader;
  BMPFileOpened                  : Boolean;
  Palette                        : Array [0..255,0..3] of Byte;
  BMPFile                        : File;

begin
  FileMode := 0; {Read only}
  {Check to see whether the file exists and can be opened}
  {$I-}
     BMPFileOpened := true;
     Assign (BMPFile, f);
     Reset (BMPFile, 1);
     if IOResult <> 0 then begin
       f := f + '.BMP';
       Assign (BMPFile, f);
       Reset (BMPFile, 1);
     end;
     Reset (BMPFile, 1);
     if IOResult <> 0 then BMPFileOpened := false;
  {$I+}
  if not BMPFileOpened then exit;

  {Read in header information}
  BlockRead (BMPFile, Header, SizeOf (Header));

  {Check to see whether we can display it}

  if (Header.bfType <> 19778) or (Header.bfReserved <> 0) or (Header.biPlanes <> 1) then begin
    {Not a valid bitmap file - don't display}
    close (BMPFile);
    exit;
  end;
  if (Header.biCompression <> 0) then begin
    {Compressed file - don't display}
    close (BMPFile);
    exit;
  end;
  if (Header.biBitCount <> 8) then begin
    {Other than 8-bit colour - don't display}
    close (BMPFile);
   exit;
  end;
  if (Header.biWidth > 320) or (Header.biHeight > 200) then begin
    {Too large - don't display}
    close (BMPFile);
    exit;
  end;

  {Load in the palette}
  BlockRead (BMPFile, Palette, 1024);

  {Divide all palette entries by 4}
  for Counter := 0 to 255 do begin
    Palette[Counter,0] := Palette[Counter, 0] shr 2;
    Palette[Counter,1] := Palette[Counter, 1] shr 2;
    Palette[Counter,2] := Palette[Counter, 2] shr 2;
    SetDAC (Counter,
            Palette [Counter, 2],
            Palette [Counter, 1],
            Palette [Counter, 0]);
  end;

   {Set appropriate position to display graphics data}
   offset := (100 + Header.biHeight shr 1) * 320 + 160 - Header.biWidth shr 1;

   {We've read no lines so far}
   lines := 0;

   {Decode and display graphics}
   while (lines < Header.biHeight) do begin
      {Read next line}
      BlockRead (BMPFile, Buffer^ [offset], Header.biWidth);
      {Move up one line on the screen}
      dec (offset, 320);
      {increase amount of lines read}
      inc (lines);
   end;
   close (BMPFile);
end;

begin
  {Check if a filename was passed as a parameter, otherwise ask for one}
  if ParamCount > 0 then
    filename := ParamStr (1)
  else begin
    write ('Enter filename:'); readln (filename);
  end;

  VGAScreen;

  {Assign memory for the buffer}
  MakeBuffer;

  {Clear the buffer}
  ClearBuffer;

  {Load the BMP File}
  LoadBMP(filename);

  {Display the buffer}
  DisplayBuffer;

  Readln;
  TextScreen;

end.